---@diagnostic disable: duplicate-set-field
--[[
   Copyright 2011 Kelly Riley (lunarwater)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

import "Compendium.Items.CompendiumItemsDB";
import "Compendium.Common.Utils";
import "Compendium.Common.UI";
import "Compendium.Extensions.AdvPaginationControl";
import "Compendium.Extensions.ProgressBar";

local rowHeight = 35;

local ITEMIDMIN = 1879049233;
local ITEMIDMAX = 1879456726;
local SCANDATA  = Compendium.Common.Utils.PluginData.Load( Turbine.DataScope.Account , "CompendiumItemScanner");

local CategoryLookup = {};
for key, value in pairs(Turbine.Gameplay.ItemCategory) do
	CategoryLookup[value] = key;
end
local QualityLookup = {};
for key, value in pairs(Turbine.Gameplay.ItemQuality) do
	QualityLookup[value] = key;
end
local typeorder = { 'Hobby', 'Skill', 'Item' }; --, 'Pet'
local types = {
				["Hobby"] = { val = Turbine.UI.Lotro.ShortcutType.Hobby, tpl = '0x%s' },
				["Skill"] = { val = Turbine.UI.Lotro.ShortcutType.Skill, tpl = '0x%s' },
				["Item"] = { val = Turbine.UI.Lotro.ShortcutType.Item, tpl = '0x0000000000000000,0x%s' }
			};
local makesc = function(type, hex, quicks)
	local shortcut = Turbine.UI.Lotro.Shortcut(types[type].val , string.format(types[type].tpl, hex ))
	quicks:SetShortcut( shortcut );
	return shortcut, type;
end

ItemBrowseControl = class( Compendium.Common.UI.CompendiumControl );
function ItemBrowseControl:Constructor()
    Compendium.Common.UI.CompendiumControl.Constructor( self );

	local fontFace = self.fontFace;

	self:ImportScanData();
    self.qualities = {
    	Common = self.white,
    	Uncommon = self.yellow,
    	Rare = self.purple,
    	Epic = self.orange,
    	Incomparable = self.lightBlue
    };

	local quals = { 'Common', 'Uncommon', 'Rare', 'Epic', 'Incomparable' };
	local qualMenu = Turbine.UI.ContextMenu();
	local qualItemList = qualMenu:GetItems();
	for i, quality in pairs(quals) do
		local item = Turbine.UI.MenuItem(quality);
    	item.Click = function(sender,args)
			self:QualityClicked(quality);
		end
    	qualItemList:Add(item);
	end
	self.qualMenu = qualMenu;

    local saveButton = Turbine.UI.Lotro.Button();
    saveButton:SetParent(self);
    saveButton:SetPosition(9,3);
    saveButton:SetSize(50,20);
    saveButton:SetText("Save");
 	saveButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	saveButton.Click = function( sender, args )
    	self:persist();
    end

    local scanButton = Turbine.UI.Lotro.Button();
    scanButton:SetParent(self);
    scanButton:SetPosition(saveButton:GetWidth() + saveButton:GetLeft(),3);
    scanButton:SetSize(100,20);
    scanButton:SetText("Scanning");
 	scanButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	scanButton.Click = function( sender, args )
		self.scanCont:SetVisible(not self.scanCont:IsVisible())
    end
	local scanIcon = Turbine.UI.Control();
    scanIcon:SetParent( scanButton );
    scanIcon:SetBackground(0x41007e19);
    scanIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
    scanIcon:SetPosition( scanButton:GetWidth() - 20, 1 );
    scanIcon:SetSize( 16, 16 );
    scanIcon.MouseClick = function( sender, args )
    	self.scanCont:SetVisible(not self.scanCont:IsVisible())
    end

    self.itemContainer=Turbine.UI.Control();
    self.itemContainer:SetParent(self);
    self.itemContainer:SetPosition(5, 30);
    self.itemContainer:SetSize(self:GetWidth() - 7,self:GetHeight() - self.itemContainer:GetTop() - 70);
    self.itemContainer:SetBackColor(Turbine.UI.Color(0,0,0)); -- this one has to stay fixed for grid to show
    self.itemContainer.ItemList=Turbine.UI.ListBox();
    self.itemContainer.ItemList:SetParent(self.itemContainer);
    self.itemContainer.ItemList:SetPosition(2,1);
    self.itemContainer.ItemList:SetSize(self.itemContainer:GetWidth()-4,self.itemContainer:GetHeight()-3);
    self.itemContainer.ItemList:SetBackColor(self.backColor);
    self.itemContainer.ItemList.VScrollBar=Turbine.UI.Lotro.ScrollBar();
    self.itemContainer.ItemList.VScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.itemContainer.ItemList.VScrollBar:SetParent(self.itemContainer.ItemList);
    self.itemContainer.ItemList.VScrollBar:SetBackColor(self.backColor);
    self.itemContainer.ItemList.VScrollBar:SetPosition(self.itemContainer:GetWidth()-16,0)
    self.itemContainer.ItemList.VScrollBar:SetWidth(12);
    self.itemContainer.ItemList.VScrollBar:SetHeight(self.itemContainer:GetHeight()-2);
    self.itemContainer.ItemList:SetVerticalScrollBar(self.itemContainer.ItemList.VScrollBar);

	local scanCont = Turbine.UI.Control();
    scanCont:SetParent(self);
    scanCont:SetPosition(5, 30);
    scanCont:SetSize(self:GetWidth() - 7,self:GetHeight() - self.itemContainer:GetTop() - 70);
    scanCont:SetBackColor(Turbine.UI.Color(0,0,0));
	scanCont:SetVisible(false)
	self.scanCont = scanCont;

	local scantop = 20;
	local cbx = Turbine.UI.Lotro.CheckBox();
    cbx:SetParent( scanCont );
    cbx:SetMultiline( true );
    cbx:SetPosition( 20, scantop );
    cbx:SetSize( 600, 20 );
    cbx:SetFont(fontFace);
    cbx:SetForeColor(self.fontColor);
    cbx:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    cbx:SetText("Skip known items from Compendium DB");
    cbx:SetChecked(self.skipknownitems);
	cbx.CheckedChanged = function(s,a)
		self.skipknownitems = s:IsChecked();
	end

	scantop = scantop + 20;
	cbx = Turbine.UI.Lotro.CheckBox();
    cbx:SetParent( scanCont );
    cbx:SetMultiline( true );
    cbx:SetPosition( 20, scantop );
    cbx:SetSize( 600, 20 );
    cbx:SetFont(fontFace);
    cbx:SetForeColor(self.fontColor);
    cbx:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    cbx:SetText("Load Name, Category, and Quality of items automatically (consumes more memory)");
    cbx:SetChecked(self.loaddetails);
	cbx.CheckedChanged = function(s,a)
		self.loaddetails = s:IsChecked();
	end

	scantop = scantop + 40;
	local fromLbl = Turbine.UI.Label();
    fromLbl:SetParent(scanCont);
    fromLbl:SetPosition(20, scantop);
	fromLbl:SetSize(80, 25);
    fromLbl:SetText('Scan Min: ');
    fromLbl:SetBackColor(self.backColor);
    fromLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    fromLbl:SetFont(fontFace);
    fromLbl:SetSelectable(true);
    fromLbl:SetForeColor(self.fontColor);

   	local from = Turbine.UI.Label();
	from:SetParent(scanCont);
   	from:SetPosition(fromLbl:GetLeft() + fromLbl:GetWidth() + 5, scantop);
	from:SetSize(90, 25);
    from:SetBackColor(self.backColor);
    from:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    from:SetFont(fontFace);
    from:SetSelectable(true);
    from:SetForeColor(self.fontColor);
	from:SetText(self.min);
	self.from = from;

	local startAtLbl = Turbine.UI.Label();
    startAtLbl:SetParent(scanCont);
    startAtLbl:SetPosition(from:GetLeft() + from:GetWidth() + 5, scantop);
	startAtLbl:SetSize(60, 25);
    startAtLbl:SetText('Start At:');
    startAtLbl:SetBackColor(self.backColor);
    startAtLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    startAtLbl:SetFont(fontFace);
    startAtLbl:SetSelectable(true);
    startAtLbl:SetForeColor(self.fontColor);

   	local startAt = Turbine.UI.Lotro.TextBox();
	startAt:SetParent(scanCont);
   	startAt:SetPosition(startAtLbl:GetLeft() + startAtLbl:GetWidth() + 5, scantop);
	startAt:SetSize(80, 25);
    startAt:SetBackColor(Turbine.UI.Color(.25,.35,.55));
    startAt:SetMultiline(false);
    startAt:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    startAt:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
    startAt:SetForeColor(self.fontColor);
    startAt:SetOutlineColor(Turbine.UI.Color(0,0,0));
	startAt:SetText(self.itemid);
	self.startAt = startAt;

	local toLbl = Turbine.UI.Label();
    toLbl:SetParent(scanCont);
    toLbl:SetPosition(startAt:GetLeft() + startAt:GetWidth() + 5, scantop);
	toLbl:SetSize(40, 25);
    toLbl:SetText('To:');
    toLbl:SetBackColor(self.backColor);
    toLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
    toLbl:SetFont(fontFace);
    toLbl:SetSelectable(true);
    toLbl:SetForeColor(self.fontColor);

   	local to = Turbine.UI.Lotro.TextBox();
	to:SetParent(scanCont);
   	to:SetPosition(toLbl:GetLeft() + toLbl:GetWidth() + 5, scantop);
	to:SetSize(80, 25);
    to:SetBackColor(Turbine.UI.Color(.25,.35,.55));
    to:SetMultiline(false);
    to:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    to:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
    to:SetForeColor(self.fontColor);
    to:SetOutlineColor(Turbine.UI.Color(0,0,0));
	to:SetText(self.max);
	self.to = to;

	scantop = scantop + 40;

	local btnCont = Turbine.UI.Control();
    btnCont:SetParent(scanCont);
    btnCont:SetBackColor(Turbine.UI.Color(0,0,0));

	local btnleft = 0
	local btn = Turbine.UI.Lotro.Button();
    btn:SetParent(btnCont);
    btn:SetPosition(btnleft, 0);
    btn:SetSize(120,20);
    btn:SetText("Default Positions");
    btn:SetEnabled(true);
 	btn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	btn.Click = function()
		from:SetText(self.min);
		to:SetText(self.max);
		startAt:SetText(self.itemid);
    end

	btnleft = btnleft + btn:GetWidth() + 5
	btn = Turbine.UI.Lotro.Button();
    btn:SetParent(btnCont);
    btn:SetPosition(btnleft, 0);
    btn:SetSize(80,20);
    btn:SetText("Clear DB");
    btn:SetEnabled(true);
 	btn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	btn.Click = function()
		self.discovereddb = {};
		self.discovereddbindex = {};
		self:ClearItems();
		self.cursor = nil;
		self:BuildCursor();
    end

	btnleft = btnleft + btn:GetWidth() + 5
    btn = Turbine.UI.Lotro.Button();
    btn:SetParent(btnCont);
    btn:SetPosition(btnleft, 0);
    btn:SetSize(90,20);
    btn:SetText("Start Scan");
    btn:SetEnabled(true);
 	btn:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	btn.Click = function( sender, args )
		self:StartScan();
    end

	btnCont:SetSize(btnleft + btn:GetWidth(), 20);
	btnCont:SetPosition(to:GetLeft() + to:GetWidth() - btnCont:GetWidth(), scantop);
	btnCont.SetEnabled = function(self, enabled)
		local ctrls = btnCont:GetControls();
		for i = 1, ctrls:GetCount() do
			ctrls:Get(i):SetEnabled(enabled);
		end
	end
	self.btnCont = btnCont;

	scantop = scantop + 40;
	local detailProgress = Turbine.UI.Label();
    detailProgress:SetSize(500, 25);
    detailProgress:SetPosition(20, scantop);
    detailProgress:SetFont(fontFace);
    detailProgress:SetForeColor(self.fontColor);
    detailProgress:SetOutlineColor(Turbine.UI.Color(0,0,0));
    detailProgress:SetParent(scanCont);
	self.detailedProgress = detailProgress;

	scantop = scantop + 25;
	local label = Turbine.UI.Label();
    label:SetSize(100, 25);
    label:SetPosition(20 + (detailProgress:GetWidth() / 2), scantop);
    label:SetFont(fontFace);
    label:SetForeColor(self.fontColor);
    label:SetOutlineColor(Turbine.UI.Color(0,0,0));
    label:SetParent(scanCont);
	self.progressLabel = label;

	scantop = scantop + 25;
	local progress = ProgressBar();
	progress:SetPosition(20, scantop);
	progress:SetWidth(detailProgress:GetWidth());
	progress:SetParent(scanCont);
	progress:SetProgress(0);
	progress.ProgressChanged = function(_, p)
		if (p >= 1) then
			self:ClearItems();
			self.cursor = nil;
			self:BuildCursor();
			label:SetText("");
			detailProgress:SetText("");
			self.btnCont:SetEnabled(true);
			local max = math.max(self.max, self.currentscan.maxid);
			self.to:SetText(max)
			startAt:SetText(math.min(self.currentscan.pos, max));
			self.scanCont:SetVisible(false);
		else
			label:SetText(math.floor(p * 100) .. "%");
			startAt:SetText(self.currentscan.pos);
		end
	end
	self.progress = progress;

    local aliasMenu = Compendium.Common.UI.ItemAliasMenu();
    aliasMenu:SetParent(self.itemContainer.ItemList);
    self.aliasMenu = aliasMenu;
    self.ClickEvent = function( item, args )
    	if args.Button == Turbine.UI.MouseButton.Right then

    		local itemrec = self.discovereddb[item.record.index];
    		if itemrec == nil then return end;
			local name = itemrec.name;
			if name == nil or name == '' then
				name = 'Unknown Item';
			end

    		-- right mouse button
    		self.aliasMenu:ShowAliasMenu({ id = DEC_HEX(itemrec.id), name = name });
    		local lh = self.itemContainer.ItemList:GetHeight();
    		local mh = self.aliasMenu:GetHeight();
    		local sx, sy = item:PointToScreen(args.X, args.Y);
    		local left, top = self.itemContainer.ItemList:PointToClient(sx, sy);

    		if (top + mh) > lh then
    			top = lh - mh;
    		end

    		self.aliasMenu:SetPosition(left, top);
    	end
    end

    local pagination = Compendium.Extensions.AdvPaginationControl();
    pagination:SetParent(self);
    pagination:SetSize(self.itemContainer:GetWidth(),20);
    pagination:SetPosition(5,self.itemContainer:GetTop() + self.itemContainer:GetHeight());
	pagination.PageChanged = function(sender,direction,records)
   		self:ClearItems();
		self:LoadItems(records);
   	end
    self.pagination = pagination;

    self:ClearItems();
	self:CalcPageSize();
	self:BuildCursor();

    self.menu = Compendium.Items.ItemCategoryMenu();
    self.menu.ClickCategory = function(categories)
    	local size = #categories;
    	if size > 0 and categories[size] == 'Custom' then
			-- do nothing
    	else
			self:AddCategories(categories);
    	end
	end

 	self.SetHeight = function(sender,height)
        if height<80 then height=80 end;
        Turbine.UI.Control.SetHeight(self,height);
        local icheight = height - self.itemContainer:GetTop() - 30;
        self.itemContainer:SetHeight(icheight);
		self.scanCont:SetHeight(icheight);
        self.itemContainer.ItemList:SetHeight(icheight - 3);
        self.itemContainer.ItemList.VScrollBar:SetHeight(icheight - 2);
        pagination:SetTop(self.itemContainer:GetTop() + self.itemContainer:GetHeight());

		-- recalculate # of rows we can show on screen per page
        self:CalcPageSize();
        if self.cursor ~= nil then
        	self.cursor:SetPageSize(self.pagesize);
	   		self:ClearItems();
			self:LoadItems(self.cursor:CurPage());
			self.pagination:UpdatePagination();
		end
    end
 	self.SetWidth = function(sender,width)
        if width<100 then width=100 end;
        Turbine.UI.Control.SetWidth(self,width);
        local icwidth = width - 7;
        self.itemContainer:SetWidth(icwidth);
		self.scanCont:SetWidth(icwidth);
        self.itemContainer.ItemList:SetWidth(icwidth - 4);
		self.itemContainer.ItemList.VScrollBar:SetLeft(icwidth-16)
		for index=1,self.itemContainer.ItemList:GetItemCount() do
			local label = self.itemContainer.ItemList:GetItem(index);
			label:SetWidth(icwidth - 13);
		end
		pagination:SetWidth(icwidth);
    end
	self.SetSize = function(sender,width, height)
		self:SetWidth(width);
		self:SetHeight(height);
	end

end

function ItemBrowseControl:AddCategories(cats)

	local catVal = '';

    for j, type in pairs( cats ) do
		--Turbine.Shell.WriteLine('cat ' .. type);
		if type ~= 'All' and _G.Compendium.Items.itemindexes[type] ~= nil then
			if catVal == '' then
				catVal = type;
			else
				catVal = catVal .. ', ' .. type;
			end
		end
	end
	self.catTb:SetText(catVal);
	self.currentRec['cat'] = catVal;

end

function ItemBrowseControl:QualityClicked(quality)
	self.qualLbl:SetBackColor(self.qualities[quality]);
	self.currentRec['qual'] = quality;
end

function ItemBrowseControl:ClearItems()
	for index=1,self.itemContainer.ItemList:GetItemCount() do
		local item = self.itemContainer.ItemList:GetItem(index);
		self:strip(item, 1);
	end
    self.itemContainer.ItemList:ClearItems();
    self.prevIdx = nil;
end

function ItemBrowseControl:BuildCursor()
	self.pagination:SetCursor(nil);
	self:ClearItems();

	if self.cursor == nil then
		self.cursor = Compendium.Common.Utils.DataCursor(self.discovereddb, self.pagesize);
	end
	self:LoadItems(self.cursor:CurPage());
	self.pagination:SetCursor(self.cursor);
end

function ItemBrowseControl:BuildDiscoveryDatabase(iterations)

	local qs = Turbine.UI.Lotro.Quickslot();
	local id = self.currentscan.pos;
	local max = self.currentscan.max;
	--Turbine.Shell.WriteLine('trying ' .. tostring(id) .. ' => ' .. tostring(iterations))

	for count = 1, iterations do
		if id > max then
			break;
		end
		local idstr = tostring(id);
		local index = self.discovereddbindex[idstr];
		local unknown = not self.knownitemsdb[idstr];
		if index == nil and unknown then
			local hex =  DEC_HEX(id); --'70000FB2'

			local s, sc, st;
			for j, type in pairs( typeorder ) do
				--Turbine.Shell.WriteLine('trying ' .. type);

				s,sc,st = pcall(makesc, type, hex, qs);
				if s then
					--Turbine.Shell.WriteLine('success ' .. type);
					break;
				else
					--Turbine.Shell.WriteLine('failed ' .. type .. ', error: ' .. sc);
					sc = nil;
					st = nil;
				end
			end

			if s and st == 'Item' then
				self.currentscan.maxid = math.max(self.currentscan.maxid, id);
				if self.loaddetails then
					local ii = qs:GetShortcut():GetItem():GetItemInfo();
					local cat = CategoryLookup[ii:GetCategory()];
					local name = ii:GetName();
					local qual = QualityLookup[ii:GetQuality()];
					table.insert(self.discovereddb, { id = id, name = name, cat = cat, qual = qual });
				else
					table.insert(self.discovereddb, { id = id });
				end
			end
		else
			self.currentscan.maxid = math.max(self.currentscan.maxid, id);
		end
		id = self.currentscan.pos + count;
	end

	self.currentscan.pos = id;
	local total = self.currentscan.total;
	local start = self.currentscan.start;
	self.detailedProgress:SetText("Scanning item " .. tostring(id - start) .. ' of ' .. tostring(total));
	--Turbine.Shell.WriteLine('id: ' .. tostring(id) .. ', start: '.. tostring(start) .. ', total: ' .. tostring(total) .. ', max: '.. tostring(max));
	return ((id - start) * 1.0) / total;
end


function ItemBrowseControl:ParsePositions()
	local p = {
		min = tonumber(self.from:GetText()) or ITEMIDMIN,
		max = tonumber(self.to:GetText()) or ITEMIDMAX,
		start = tonumber(self.startAt:GetText()) or ITEMIDMAX
	};
	if p.start > p.max then
		p.start = p.max;
	end
	if p.min > p.max then
		p.min = ITEMIDMIN;
	end
	return p;
end

function ItemBrowseControl:StartScan()
	self.__bailout = 0;
	self.btnCont:SetEnabled(false);
	self:ClearItems();
	self.cursor = nil;
	self.currentscan = self:ParsePositions();
	self.currentscan.max = self.currentscan.max + 1000;
	self.currentscan.pos = self.currentscan.start;
	self.currentscan.total = self.currentscan.max - self.currentscan.start;
	self.currentscan.maxid = 0;
	self.knownitemsdb = {};
	if self.skipknownitems then
		for i,item in ipairs(_G.Compendium.Items.itemstable) do
			-- if math.fmod(i, 1000) == 0 then
			-- 	Turbine.Shell.WriteLine(tostring(i) .. ' item : ' .. item.id);
			-- end
			self.knownitemsdb[tostring(tonumber(item.id, 16))] = true;
		end
	end
	self.progress:Start(function(workToDo)
		return self:BuildDiscoveryDatabase(workToDo)
	end, 10, 1);
end

function ItemBrowseControl:ImportScanData()
	if SCANDATA == nil then
		SCANDATA = {
			min = ITEMIDMIN,
			max = ITEMIDMAX,
			itemid = ITEMIDMIN,
			discovereddb = {},
			loaddetails = false,
			skipknownitems = true
		};
	end
	self.min = math.min(SCANDATA.min, ITEMIDMIN);
	self.max = math.max(SCANDATA.max, ITEMIDMAX);
	self.discovereddbindex = {};
	-- adjust min & max to loaded scan data bounds
	--local start = Turbine.Engine.GetGameTime();
	for i,item in pairs(SCANDATA.discovereddb) do
		local number = tonumber(item.id, 10);
		self.min = math.min(self.min, number);
		self.max = math.max(self.max, number);
		self.discovereddbindex[number] = i;
	end
	--Turbine.Shell.WriteLine("Building Item Scan Index took : " .. tostring(Turbine.Engine.GetGameTime() - start) .. " seconds");
	self.itemid = math.min(math.max(SCANDATA.itemid, self.min), self.max);
	self.discovereddb = SCANDATA.discovereddb;
	-- use item lookup to get name / quality / category of items
	self.loaddetails = SCANDATA.loaddetails;
	if self.loaddetails == nil then
		self.loaddetails = false;
	end
	-- skip compendium db items
	self.skipknownitems = SCANDATA.skipknownitems;
	if self.skipknownitems == nil then
		self.skipknownitems = true
	end
	SCANDATA = nil;
end

function ItemBrowseControl:persist()
	local pos = self:ParsePositions();
	Compendium.Common.Utils.PluginData.Save( Turbine.DataScope.Account, "CompendiumItemScanner", {
		  		min = pos.min,
		  		max = pos.max,
		  		itemid = pos.start,
		  		discovereddb = self.discovereddb,
				loaddetails = self.loaddetails,
				skipknownitems = self.skipknownitems
		  	});
end

function DEC_HEX(IN)
    return string.upper(string.format('%x', IN));
end

local makesc = function(hex, quicks)
	local shortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Item , string.format('0x0000000000000000,0x%s', hex ))
	quicks:SetShortcut( shortcut );
	quicks.DragDrop=function()
		--quicks:SetShortcut(shortcut);
	end
	return shortcut, type;
end

function ItemBrowseControl:LoadItems(records)

    local bgColor = self.itemContainer.ItemList:GetBackColor();
    local width = self.itemContainer.ItemList:GetWidth();
	local baseIndex = self.cursor.offset;

	local firstQs = nil;
    for i,rec in pairs(records) do
    	local hex = DEC_HEX(rec.id);
        local name = rec.id .. ' ' .. hex;
        local default = true;
		if rec.name ~= nil then
			name = rec.name;
			default = false;
		end

        local itemRow = Turbine.UI.Control();
        itemRow:SetSize(width - 13, rowHeight);
        itemRow:SetBackColor(bgColor);

        local left = 0;
        local width = itemRow:GetWidth();

		local qs = Turbine.UI.Lotro.Quickslot();
		qs:SetSize(35,35);
		qs:SetVisible(true);
        qs:SetAllowDrop(false);
        qs:SetEnabled(false);
        qs:SetUseOnRightClick(false);
        local s = pcall(makesc, hex, qs);
        if s then
			qs:SetParent(itemRow);
			qs:SetPosition(left, 0);
			width = width - 35;
			left = 36;
	        qs.record = { id = rec['id'], index = (i + baseIndex - 1) };
			qs.MouseDown=function(s,a)
				-- hack to prevent real item from being consumed by a click
				local sc = qs:GetShortcut();
				qs:SetShortcut(nil);
				qs:SetShortcut(sc);
				self.ClickEvent(qs, a);
			end
		end

        local label = Turbine.UI.Label();
        label:SetSize(20, 20);
        label:SetParent(itemRow);
        label:SetPosition(left,12);
        label:SetText(self:FormatItem(hex, "?"));
        label:SetBackColor(bgColor);
        --label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
        label:SetFont(Turbine.UI.Lotro.Font.TrajanPro28);
        label:SetSelectable(true);
        label:SetForeColor(self.fontColor);

        left = left + 25;

        local qualCont = Turbine.UI.Control();
	    qualCont:SetParent(itemRow);
	    qualCont:SetPosition(left,8);
	    qualCont:SetSize(20,20);
	    qualCont:SetBackColor(Turbine.UI.Color(0.18,0.19,0.29));
        local qualLbl = Turbine.UI.Label();
	    qualLbl:SetParent(qualCont);
	    qualLbl:SetPosition(2,2);
	    qualLbl:SetSize(16,16);
	    local qual = 'Common';
	    if rec.qual ~= nil then qual = rec.qual end;
	    pcall(function() qualLbl:SetBackColor(self.qualities[qual]) end);
        qualLbl.MouseClick = function()
	    	self.currentRec = rec;
	    	self.qualLbl = qualLbl;
	    	self.qualMenu:ShowMenu();
        end

        left = left + qualCont:GetWidth() + 2;

		width = width - left - 90;
        local ntbw = math.ceil(.40 * width);
        local ctbw = math.ceil(.60 * width);

	   	local nameTb = Turbine.UI.Lotro.TextBox();
		nameTb:SetParent(itemRow);
		nameTb:SetSize(ntbw,25);
		nameTb:SetVisible(true);
		nameTb:SetPosition(left,5);
	    nameTb:SetBackColor(Turbine.UI.Color(.25,.35,.55));
	    nameTb:SetMultiline(false);
	    nameTb:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	    nameTb:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	    nameTb:SetForeColor(self.fontColor);
	    nameTb:SetOutlineColor(Turbine.UI.Color(0,0,0));
	    if not default then nameTb:SetText(name) end;
	    nameTb:SetSelectable(true);
		nameTb.FocusLost=function(s)
			rec['name'] = nameTb:GetText();
		end

		left = left + nameTb:GetWidth() + 2;
	   	local levelTb = Turbine.UI.Lotro.TextBox();
		levelTb:SetParent(itemRow);
		levelTb:SetSize(30,25);
		levelTb:SetVisible(true);
		levelTb:SetPosition(left,5);
	    levelTb:SetBackColor(Turbine.UI.Color(.25,.35,.55));
	    levelTb:SetMultiline(false);
	    levelTb:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	    levelTb:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	    levelTb:SetForeColor(self.fontColor);
	    levelTb:SetOutlineColor(Turbine.UI.Color(0,0,0));
	    if rec.level ~= nil then levelTb:SetText(rec.level) end;
	    levelTb:SetSelectable(true);
		levelTb.FocusLost=function(s)
			rec['level'] = levelTb:GetText();
		end

		left = left + levelTb:GetWidth() + 2;


	    local catIcon = Turbine.UI.Control();
	    catIcon:SetParent( itemRow );
	    catIcon:SetBackground(0x41007e19);
	    catIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	    catIcon:SetPosition( left, 7 );
	    catIcon:SetSize( 16, 16 );


		left = left + 18;

	   	local catTb = Turbine.UI.Lotro.TextBox();
		catTb:SetParent(itemRow);
		catTb:SetSize(ctbw,25);
		catTb:SetVisible(true);
		catTb:SetPosition(left,5);
	    catTb:SetBackColor(Turbine.UI.Color(.25,.35,.55));
	    catTb:SetMultiline(true);
	    catTb:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	    catTb:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	    catTb:SetForeColor(self.fontColor);
	    catTb:SetOutlineColor(Turbine.UI.Color(0,0,0));
	    if rec.cat ~= nil then catTb:SetText(rec.cat) end;
	    catTb:SetSelectable(true);
		catTb.FocusLost=function(s)
			rec['cat'] = catTb:GetText();
		end
	    catIcon.MouseClick = function( sender, args )
	    	self.currentRec = rec;
	    	self.catTb = catTb;
	    	self.menu:ShowMenu();
	    end

	    left = left + ctbw + 2;

        local idLbl = Turbine.UI.Label();
        idLbl:SetSize(70, 20);
        idLbl:SetParent(itemRow);
        idLbl:SetPosition(left,7);
        idLbl:SetText(tostring(rec.id));
        idLbl:SetBackColor(bgColor);
        idLbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
        idLbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
        idLbl:SetSelectable(true);
        idLbl:SetForeColor(self.fontColor);

		itemRow.SetWidth = function(s,w)
			Turbine.UI.Control.SetWidth(s, w);
			local rest = w - 35 - 25 - 25 - 32 - 90;
			local ntbw = math.ceil(.40 * rest);
        	local ctbw = math.ceil(.60 * width);
			nameTb:SetWidth(ntbw);
			levelTb:SetLeft(ntbw + nameTb:GetLeft() + 2);
			catIcon:SetLeft(30 + levelTb:GetLeft() + 2)
			catTb:SetWidth(ctbw);
			catTb:SetLeft(16 + catIcon:GetLeft() + 2);
			idLbl:SetLeft(catTb:GetLeft() + ctbw + 2);
		end

        self.itemContainer.ItemList:AddItem(itemRow);

    end

end

function ItemBrowseControl:FormatItem(id,  name)
	local item = '';
	if id ~= nil then
		item = string.format(self.itemExampleTpl,id,name);
	else
		item = '['..name..']';
	end
	return item;
end

function ItemBrowseControl:GetExtensionName()
	return "Item Scanner";
end


function ItemBrowseControl:CalcPageSize()
	local listHeight = self.itemContainer.ItemList:GetHeight();
	self.pagesize = math.floor(listHeight / rowHeight);
end